# Copyright (c) 2009-2010 Denis Bilenko. See LICENSE for details.
import sys

noisy = True

__all__ = [
            'patch_all',
            'patch_socket',
          ]

def patch_socket(aggressive=False):
    """Replace the standard socket object with meinheld's cooperative sockets.
    """
    from meinheld import msocket
    _socket = __import__('socket')
    _socket.patched = True
    _socket.socket = msocket.socket
    _socket.SocketType = msocket.SocketType
    if hasattr(msocket, 'socketpair'):
        _socket.socketpair = msocket.socketpair
    if hasattr(msocket, 'fromfd'):
        _socket.fromfd = msocket.fromfd

# def patch_ssl():
    # try:
        # _ssl = __import__('ssl')
    # except ImportError:
        # return
    # from meinheld.ssl import SSLSocket, wrap_socket, get_server_certificate, sslwrap_simple
    # _ssl.SSLSocket = SSLSocket
    # _ssl.wrap_socket = wrap_socket
    # _ssl.get_server_certificate = get_server_certificate
    # _ssl.sslwrap_simple = sslwrap_simple



def patch_all(socket=True, aggressive=True):
    """Do all of the default monkey patching (calls every other function in this module."""
    # order is important
    if socket:
        patch_socket(aggressive=aggressive)
    # if ssl:
        # patch_ssl()





import datetime
import logging
logging.Logger.manager.emittedNoHandlerWarning = 1
import os
import sys
import traceback

import fcntl

def _close_on_exec(fd):
    flags = fcntl.fcntl(fd, fcntl.F_GETFD)
    flags |= fcntl.FD_CLOEXEC
    fcntl.fcntl(fd, fcntl.F_SETFD, flags)

class SafeAtoms(dict):

    def __init__(self, atoms):
        dict.__init__(self)
        for key, value in atoms.items():
            self[key] = value.replace('"', '\\"')

    def __getitem__(self, k):
        if k.startswith("{"):
            kl = k.lower()
            if kl in self:
                return super(SafeAtoms, self).__getitem__(kl)
            else:
                return "-"
        if k in self:
            return super(SafeAtoms, self).__getitem__(k)
        else:
            return '-'

class Logger(object):

    LOG_LEVELS = {
        "critical": logging.CRITICAL,
        "error": logging.ERROR,
        "warning": logging.WARNING,
        "info": logging.INFO,
        "debug": logging.DEBUG
    }

    error_fmt = r"%(asctime)s [%(process)d] [%(levelname)s] %(message)s"
    datefmt = r"%Y-%m-%d %H:%M:%S"

    access_log_format = '"%(h)s %(l)s %(u)s [%(t)s] "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
    access_fmt = "%(message)s"

    def __init__(self, cfg=None):
        self.error_log = logging.getLogger("meinheld.error")
        self.access_log = logging.getLogger("meinheld.access")
        self.error_handlers = []
        self.access_handlers = []
        self.setup(cfg)

    def setup(self, cfg):
        self.error_log.setLevel(logging.ERROR)
        self.access_log.setLevel(logging.INFO)

        self._set_handler(self.error_log, logging.Formatter(self.error_fmt, self.datefmt))
        self._set_handler(self.access_log, fmt=logging.Formatter(self.access_fmt))

    def critical(self, msg, *args, **kwargs):
        self.error_log.critical(msg, *args, **kwargs)

    def error(self, exc, val, tb):
        from traceback import format_exception
        msg = ''.join(format_exception(exc, val, tb))
        self.error_log.error(msg)

    def warning(self, msg, *args, **kwargs):
        self.error_log.warning(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        self.error_log.info(msg, *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        self.error_log.debug(msg, *args, **kwargs)

    def exception(self, msg, *args):
        self.error_log.exception(msg, *args)

    def log(self, lvl, msg, *args, **kwargs):
        if isinstance(lvl, basestring):
            lvl = self.LOG_LEVELS.get(lvl.lower(), logging.INFO)
        self.error_log.log(lvl, msg, *args, **kwargs)

    def access(self, environ):
        """ Seee http://httpd.apache.org/docs/2.0/logs.html#combined
        for format details
        """

        # status = resp.status.split(None, 1)[0]
        atoms = {
                'h': environ.get('REMOTE_ADDR', '-'),
                'l': '-',
                'u': '-', # would be cool to get username from basic auth header
                't': environ.get('LOCAL_TIME', '-'),
                'r': "%s %s %s" % (environ.get('REQUEST_METHOD', '-'), environ.get('PATH_INFO', '-'), environ.get('SERVER_PROTOCOL', 'HTTP/1.0')),
                's': str(environ.get('STATUS_CODE', '-')),
                'b': str(environ.get('SEND_BYTES', '-')),
                'f': environ.get('HTTP_REFERER', '-'),
                'a': environ.get('HTTP_USER_AGENT', '-'),
                'T': str(environ.get('REQUEST_TIME', 1) /100),
                'D': str(environ.get('REQUEST_TIME', 1)),
                'p': "<%s>" % os.getpid()
                }
        # add request headers
        
        for k, v in environ.items():
            if k.startswith('HTTP_'):
                #header
                header = "{%s}i" % k[5:].lower()
                atoms[header] = v

        # atoms.update(dict([("{%s}i" % k.lower(),v) for k, v in req_headers]))

        # # add response headers
        # atoms.update(dict([("{%s}o" % k.lower(),v) for k, v in resp.headers]))

        # # wrap atoms:
        # # - make sure atoms will be test case insensitively
        # # - if atom doesn't exist replace it by '-'
        
        safe_atoms = SafeAtoms(atoms)

        try:
            # self.access_log.info(self.access_log_format % safe_atoms)
            self.access_log.info(self.access_log_format % safe_atoms)
        except:
            self.error(*sys.exc_info())


    def reopen_files(self):
        for log in (self.error_log, self.access_log):
            for handler in log.handlers:
                if isinstance(handler, logging.FileHandler):
                    handler.acquire()
                    try:
                        if handler.stream:
                            handler.stream.close()
                            handler.stream = open(handler.baseFilename,
                                    handler.mode)
                    finally:
                        handler.release()

    def close_on_exec(self):
        for log in (self.error_log, self.access_log):
            for handler in log.handlers:
                if isinstance(handler, logging.FileHandler):
                    handler.acquire()
                    try:
                        if handler.stream:
                            _close_on_exec(handler.stream.fileno())
                    finally:
                        handler.release()


    def _get_handler(self, log):
        for h in log.handlers:
            if getattr(h, "_meinheld", False) == True:
                return h

    def _set_handler(self, log, fmt):
        h = self._get_handler(log)
        if h:
            log.handlers.remove(h)

        h = logging.StreamHandler()
        h._meinheld = True
        h.setFormatter(fmt)
        log.addHandler(h)

def _error(self, exc, val, tb):
    from traceback import format_exception
    msg = ''.join(format_exception(exc, val, tb))
    self.error_log.error(msg)

def _access(self, environ):
    """ Seee http://httpd.apache.org/docs/2.0/logs.html#combined
    for format details
    """

    # status = resp.status.split(None, 1)[0]
    atoms = {
            'h': environ.get('REMOTE_ADDR', '-'),
            'l': '-',
            'u': '-', # would be cool to get username from basic auth header
            't': "[%s]" % environ.get('LOCAL_TIME', '-'),
            'r': "%s %s %s" % (environ.get('REQUEST_METHOD', '-'), environ.get('PATH_INFO', '-'), environ.get('SERVER_PROTOCOL', 'HTTP/1.0')),
            's': str(environ.get('STATUS_CODE', '-')),
            'b': str(environ.get('SEND_BYTES', '-')),
            'f': environ.get('HTTP_REFERER', '-'),
            'a': environ.get('HTTP_USER_AGENT', '-'),
            'T': str(environ.get('REQUEST_TIME', 1) /100),
            'D': str(environ.get('REQUEST_TIME', 1)),
            'p': "<%s>" % os.getpid()
            }
    # add request headers
    
    for k, v in environ.items():
        if k.startswith('HTTP_'):
            #header
            header = "{%s}i" % k[5:].lower()
            atoms[header] = v

    # atoms.update(dict([("{%s}i" % k.lower(),v) for k, v in req_headers]))

    # # add response headers
    # atoms.update(dict([("{%s}o" % k.lower(),v) for k, v in resp.headers]))

    # # wrap atoms:
    # # - make sure atoms will be test case insensitively
    # # - if atom doesn't exist replace it by '-'
    
    safe_atoms = SafeAtoms(atoms)

    try:
        msg = self.cfg.access_log_format % safe_atoms
        self.access_log.info(msg)
    except:
        self.error(*sys.exc_info())

logger = Logger()
from meinheld import server
server.set_access_logger(logger)
server.set_error_logger(logger)




import sys
import types
from gunicorn.workers.base import Worker
from gunicorn.glogging import Logger
from meinheld import server
from meinheld.mlogging import _access, _error
import os

def is_py3():
    return sys.hexversion >=  0x3000000

if not is_py3():


    class MeinheldWorker(Worker):

        def __init__(self, *args, **kwargs):
            Worker.__init__(self, *args, **kwargs)
            Logger.access = _access
            Logger.error = _error

            server.set_access_logger(self.log)
            server.set_error_logger(self.log)
        
        def watchdog(self):
            self.notify()

            if self.ppid != os.getppid():
                self.log.info("Parent changed, shutting down: %s" % self)
                server.stop(int(self.timeout))

        def run(self):
            
            if hasattr(self, "sockets"):
                fds = [s.fileno() for s in self.sockets]
            else:
                fds = [self.socket.fileno()]

            server.set_keepalive(self.cfg.keepalive)
            server.set_picoev_max_fd(self.cfg.worker_connections)
            
            server.set_fastwatchdog(self.tmp.fileno(), self.ppid, int(self.timeout))
            #server.set_watchdog(self.watchdog)
            
            server.set_listen_socket(fds)
            server.run(self.wsgi)

        def handle_quit(self, sig, frame):
            server.stop(int(self.timeout))

        def handle_exit(self, sig, frame):
            server.stop()
            sys.exit(0)




import collections
import string
import struct
from base64 import b64encode

import sys
def is_py3():
    return sys.hexversion >=  0x3000000

if is_py3():
    from itertools import cycle
else:
    from itertools import cycle
    from itertools import imap as map
    from itertools import izip as zip
import random
import socket

try:
    from hashlib import md5, sha1
except ImportError: #pragma NO COVER
    from md5 import md5
    from sha import sha as sha1

from meinheld import server, patch
from meinheld.common import Continuation, CLIENT_KEY, CONTINUATION_KEY
patch.patch_socket()

import socket

def _wsgi_to_bytes(s):
    if isinstance(s, bytes):
        return s
    else:
        return s.encode('iso-8859-1')

def _extract_comma(value):
    return [x.strip() for x in value.split(',')]


class WebSocketMiddleware(object):

    def __init__(self, app):
        self.app = app

    def _extract_number(self, value):
        out = ""
        spaces = 0
        for char in value:
            if char in string.digits:
                out += char
            elif char == " ":
                spaces += 1
        return int(out) / spaces

    def setup(self, environ):
        protocol_version = None
        if not ("Upgrade" in _extract_comma(environ.get('HTTP_CONNECTION','')) and
                environ.get('HTTP_UPGRADE','').lower() == 'websocket'):
            return 
        if 'HTTP_SEC_WEBSOCKET_KEY' in environ:
            protocol_version = environ['HTTP_SEC_WEBSOCKET_VERSION']  # RFC 6455
            if protocol_version in ('13',):  #skip version 4,5,6,7,8
                protocol_version = int(protocol_version)
            else:
                # Unknown
                raise NotImplementedError("Not Supported")
        else:
            raise NotImplementedError("Not Supported")

        # Get the underlying socket and wrap a WebSocket class around it
        client = environ[CLIENT_KEY]
        sock = socket.fromfd(client.get_fd(), socket.AF_INET, socket.SOCK_STREAM)
        ws = WebSocket(sock, environ, protocol_version)
       
        # If it's new-version, we need to work out our challenge response
        key1 = _wsgi_to_bytes(environ['HTTP_SEC_WEBSOCKET_KEY'])
        key2 = _wsgi_to_bytes('258EAFA5-E914-47DA-95CA-C5AB0DC85B11')
        digest = sha1(key1 + key2).digest()
        response = b64encode(digest).strip()
        if is_py3():
            response = response.decode("iso-8859-1")
       
        # Start building the response
        location = 'ws://%s%s%s' % (
            environ.get('HTTP_HOST'), 
            environ.get('SCRIPT_NAME'), 
            environ.get('PATH_INFO')
        )
        qs = environ.get('QUERY_STRING')
        if qs:
            location += '?' + qs
        if protocol_version == 13:
            handshake_reply = ("HTTP/1.1 101 Switching Protocols\r\n"
                               "Upgrade: websocket\r\n"
                               "Connection: Upgrade\r\n"
                               "Origin: %s\r\n"
                               "Sec-WebSocket-Accept: %s\r\n"
                               "\r\n"% (
                    environ.get('HTTP_ORIGIN'),
                    response))
            if 'HTTP_SEC_WEBSOCKET_PROTOCOL' in environ:
                handshake_reply += 'Sec-WebSocket-Protocol: %s\r\n' % environ.get('HTTP_SEC_WEBSOCKET_PROTOCOL')
        else: #pragma NO COVER
            raise ValueError("Unknown WebSocket protocol version.") 

        sock.sendall(_wsgi_to_bytes(handshake_reply))
        environ['wsgi.websocket'] = ws
        return True

    def spawn_call(self, environ, start_response):
        result = self.setup(environ)
        response = None
        try:
            response = self.app(environ, start_response)
            return response
        finally:
            if result and response != -1:
                ws = environ.pop('wsgi.websocket')
                ws._send_closing_frame(True)
                client = environ[CLIENT_KEY]
                client.set_closed(1)

    def __call__(self, environ, start_response):
        client = environ[CLIENT_KEY]
        c = Continuation(client)
        environ[CONTINUATION_KEY] = c

        return self.spawn_call(environ, start_response)

class WebSocketWSGI(object):

    def __init__(self, handler):
        self.handler = handler
        self.protocol_version = None

    def __call__(self, environ, start_response):
        if not ("Upgrade" in _extract_comma(environ.get('HTTP_CONNECTION','')) and
                environ.get('HTTP_UPGRADE','').lower() == 'websocket'):
            # need to check a few more things here for true compliance
            start_response('400 Bad Request', [('Connection','close')])
            return [""]
    
        if 'HTTP_SEC_WEBSOCKET_KEY' in environ:
            protocol_version = environ['HTTP_SEC_WEBSOCKET_VERSION']  # RFC 6455
            if protocol_version in ('13',):  #skip version 4,5,6,7,8
                protocol_version = int(protocol_version)
            else:
                # Unknown
                raise NotImplementedError("Not Supported")
        else:
            # Unknown
            raise NotImplementedError("Not Supported")

        # Get the underlying socket and wrap a WebSocket class around it
        client = environ[CLIENT_KEY]
        sock = server._get_socket_fromfd(client.get_fd(), socket.AF_INET,
                socket.SOCK_STREAM)
        ws = WebSocket(sock, environ, self.protocol_version)

        # If it's new-version, we need to work out our challenge response
        key1 = _wsgi_to_bytes(environ['HTTP_SEC_WEBSOCKET_KEY'])
        key2 = _wsgi_to_bytes('258EAFA5-E914-47DA-95CA-C5AB0DC85B11)')
        digest = sha1(key1 + key2).digest()
        response = b64encode(digest).strip()
        if is_py3():
            response = response.decode("iso-8859-1")

        # Start building the response
        location = 'ws://%s%s%s' % (
            environ.get('HTTP_HOST'), 
            environ.get('SCRIPT_NAME'), 
            environ.get('PATH_INFO')
        )
        qs = environ.get('QUERY_STRING')
        if qs:
            location += '?' + qs
        if protocol_version == 13:
            handshake_reply = ("HTTP/1.1 101 Switching Protocols\r\n"
                               "Upgrade: websocket\r\n"
                               "Connection: Upgrade\r\n"
                               "Origin: %s\r\n"
                               "Sec-WebSocket-Accept: %s\r\n"
                               "\r\n"% (
                    environ.get('HTTP_ORIGIN'),
                    response))
            if 'HTTP_SEC_WEBSOCKET_PROTOCOL' in environ:
                handshake_reply += 'Sec-WebSocket-Protocol: %s\r\n' % environ.get('HTTP_SEC_WEBSOCKET_PROTOCOL')
        else: #pragma NO COVER
            raise ValueError("Unknown WebSocket protocol version.") 
        
        r = sock.sendall(_wsgi_to_bytes(handshake_reply))
        self.handler(ws)
        # Make sure we send the closing frame
        ws._send_closing_frame(True)
        # use this undocumented feature of eventlet.wsgi to ensure that it
        # doesn't barf on the fact that we didn't call start_response
        return [""]

    def _extract_number(self, value):
        """
        Utility function which, given a string like 'g98sd  5[]221@1', will
        return 9852211. Used to parse the Sec-WebSocket-Key headers.
        """
        out = ""
        spaces = 0
        for char in value:
            if char in string.digits:
                out += char
            elif char == " ":
                spaces += 1
        return int(out) / spaces

class WebSocket(object):
    """A websocket object that handles the details of
    serialization/deserialization to the socket.
    
    The primary way to interact with a :class:`WebSocket` object is to
    call :meth:`send` and :meth:`wait` in order to pass messages back
    and forth with the browser.  Also available are the following
    properties:
    
    path
        The path value of the request.  This is the same as the WSGI PATH_INFO variable, but more convenient.
    protocol
        The value of the Websocket-Protocol header.
    origin
        The value of the 'Origin' header.
    environ
        The full WSGI environment for this request.

    """
    def __init__(self, sock, environ, version=76):
        """
        :param socket: The eventlet socket
        :type socket: :class:`eventlet.greenio.GreenSocket`
        :param environ: The wsgi environment
        :param version: The WebSocket spec version to follow (default is 76)
        """
        self.socket = sock
        self.origin = environ.get('HTTP_ORIGIN')
        self.protocol = environ.get('HTTP_WEBSOCKET_PROTOCOL')
        self.path = environ.get('PATH_INFO')
        self.environ = environ
        self.version = version
        self.websocket_closed = False
        self._buf = b""
        self._msgs = collections.deque()
        #self._sendlock = semaphore.Semaphore()

    def _pack_message(self, message):
        """Pack the message inside ``00`` and ``FF``

        As per the dataframing section (5.3) for the websocket spec
        """
        if self.version in (13,):
            # payload
            opcode = 2
            if not isinstance(message, bytes):
                payload = message.encode('utf-8')
                opcode = 1
            elif not isinstance(message, str):
                payload = str(message)

            # header(fin,maskflag,opcode,length)
            fin = 0x80  #0x80:fin, 0:continuation
            mask = 0  #0:unmasked, 0x80:masked
            length = len(payload)
            if length < 126:
                header = struct.pack(">BB", fin|opcode, mask|length)
            elif 126 <= length <= 0xffff:
                header = struct.pack(">BBH", fin|opcode, mask|126, length)
            elif 0xffff < length <= 0xffffffffffffffff:
                header = struct.pack(">BBQ", fin|opcode, mask|127, length)
            else:
                #TODO: partial packet
                raise ValueError("Can't send over 64bit length. (partial packet are not supported)") 

            # maskdata, masked-payload
            maskdata = b''
            if mask:
                maskdata = struct.pack(">I", random.randint(0,0xffffffff))
                masklist = cycle(ord(x) for x in maskdata)
                payload = b''.join(chr(ord(d)^m) for d,m in zip(payload, masklist))

            packed = header + maskdata + payload
        else:
            raise ValueError("Unknown WebSocket protocol version.") 

        return packed

    def _parse_messages(self):
        """ Parses for messages in the buffer *buf*.  It is assumed that
        the buffer contains the start character for a message, but that it
        may contain only part of the rest of the message.

        Returns an array of messages, and the buffer remainder that
        didn't contain any full messages."""
        msgs = []
        buf = self._buf
        if self.version in (13,):
            idx = 0
            while buf:
                # b1, b2 = struct.unpack('>BB', buf[idx:idx+1])
                if is_py3():
                    b1, b2 = buf[idx], buf[idx+1]
                else:
                    b1, b2 = ord(buf[idx]), ord(buf[idx+1])

                idx += 2
                fin = bool(b1 & 0x80)  #TODO with opcode==0
                opcode = b1 & 0x0f
                mask = bool(b2 & 0x80)
                length = (b2 & 0x7f)
                if length == 126:
                    length = struct.unpack('>H', buf[idx:idx+2])
                    idx += 2
                elif length == 127:
                    length = struct.unpack('>Q', buf[dx::idx+8])
                    idx += 8

                if mask:
                    maskdata = buf[idx:idx+4]
                    idx += 4

                data = buf[idx:idx+length]
                idx += length

                if mask:
                    if is_py3():
                        data = ''.join(chr(d^m) for d,m in zip(data, cycle(maskdata)))
                    else:
                        data = b''.join(chr(d^m) for d,m in zip(
                                                        map(ord, data),
                                                        cycle(map(ord, maskdata))
                                                        ))
                if opcode == 0:  #continuation
                    pass  #TODO with fin
                elif opcode == 1:  #text
                    if is_py3():
                        data = data.encode('iso-8859-1').decode('utf-8')
                    else:
                        data = data.decode('utf-8', 'replace')
                elif opcode == 2:  #binary
                    pass
                elif opcode == 8:  #close
                    self.websocket_closed = True
                    #TODO process 2byte close status
                    break
                elif opcode == 9:  #ping
                    pass  #TODO
                elif opcode == 10: #pong
                    pass  #TODO
                else:
                    raise ValueError("Don't understand how to parse this type of message: %r" % buf)
                msgs.append(data)
                buf = buf[idx:]
        else:
            raise ValueError("Unknown WebSocket protocol version.") 

        self._buf = buf
        return msgs
    
    def send(self, message):
        """Send a message to the browser.  *message* should be
        convertable to a string; unicode objects should be encodable
        as utf-8."""
        packed = self._pack_message(message)
        # if two greenthreads are trying to send at the same time
        # on the same socket, sendlock prevents interleaving and corruption
        
        #self._sendlock.acquire()
        #try:
        return self.socket.sendall(packed)
        #finally:
        #    self._sendlock.release()

    def wait(self):
        """Waits for and deserializes messages. Returns a single
        message; the oldest not yet processed."""
        while not self._msgs:
            # Websocket might be closed already.
            if self.websocket_closed:
                return None
            # no parsed messages, must mean buf needs more data
            delta = self.socket.recv(8096)
            if delta == b'':
                return None
            self._buf += delta
            msgs = self._parse_messages()
            self._msgs.extend(msgs)
        return self._msgs.popleft()

    def _send_closing_frame(self, ignore_send_errors=False):
        """Sends the closing frame to the client, if required."""
        if self.version == 76 and not self.websocket_closed:
            try:
                self.socket.send("\xff\x00")
            except IOError:
                # Sometimes, like when the remote side cuts off the connection,
                # we don't care about this.
                if not ignore_send_errors: #pragma NO COVER
                    raise
            self.websocket_closed = True

    def close(self):
        """Forcibly close the websocket; generally it is preferable to
        return from the handler method."""
        self._send_closing_frame()
        self.socket.shutdown(True)
        self.socket.close()





try:
    from setuptools import Extension, setup
except ImportError:
    from distutils.core import Extension, setup

import os
import sys
import os.path
import platform
import fnmatch


develop = False
# develop = True 

def read(name):
    return open(os.path.join(os.path.dirname(__file__), name)).read()

def check_platform():
    if "posix" not in os.name:
        print("Are you really running a posix compliant OS ?")
        print("Be posix compliant is mandatory")
        sys.exit(1)

def check_pypy():
    return "PyPy" in sys.version

def get_picoev_file():
    poller_file = None

    if "Linux" == platform.system():
        poller_file = 'meinheld/server/picoev_epoll.c'
    elif "Darwin" == platform.system():
        poller_file = 'meinheld/server/picoev_kqueue.c'
    elif "FreeBSD" == platform.system():
        poller_file = 'meinheld/server/picoev_kqueue.c'
    else:
        print("Sorry, not support .")
        sys.exit(1)
    return poller_file

def get_sources(path, ignore_files):
    src = []
    for root, dirs , files in os.walk(path):
        for file in files:
            src_path = os.path.join(root, file)
            #ignore = reduce(lambda x, y: x or y, [fnmatch.fnmatch(src_path, i) for i in ignore_files])
            ignore = [i for i in ignore_files if  fnmatch.fnmatch(src_path, i)]
            if not ignore and src_path.endswith(".c"):
                src.append(src_path)
    return src

check_platform()
pypy = check_pypy()

if pypy:
    define_macros=[
            ("HTTP_PARSER_DEBUG", "0") ]
    install_requires=[]
else:
    define_macros=[
            ("WITH_GREENLET",None),
            ("HTTP_PARSER_DEBUG", "0") ]
    install_requires=['greenlet==0.4.0']

if develop:
    define_macros.append(("DEVELOP",None))

sources = get_sources("meinheld", ["*picoev_*"])
sources.append(get_picoev_file())

library_dirs=[]
#TODO set python include dirs
include_dirs=[]

setup(name='meinheld',
    version="0.5.4",
    description="High performance asynchronous Python WSGI Web Server",
    long_description=read('README.rst'),
    author='yutaka matsubara',
    author_email='yutaka.matsubara@gmail.com',
    url='http://meinheld.org',
    license='BSD',
    platforms='Linux, BSD, Darwin',
    packages= ['meinheld'],
    install_requires=install_requires,

    entry_points="""

    [gunicorn.workers]
    gunicorn_worker=meinheld.gmeinheld:MeinheldWorker
    """,
    ext_modules = [
        Extension('meinheld.server',
            sources=sources,
            include_dirs=include_dirs,
            library_dirs=library_dirs,
            # libraries=["profiler"],
            # extra_compile_args=[""],
            define_macros=define_macros
        )],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: BSD :: FreeBSD',
        'Programming Language :: C',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Server'
    ],
)




# Copyright (c) 2005-2006, Bob Ippolito
# Copyright (c) 2007, Linden Research, Inc.
# Copyright (c) 2009-2010 Denis Bilenko
# Copyright (c) 2010 Yutaka Matsubara
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Cooperative socket module.

This module provides socket operations and some related functions.
The API of the functions and classes matches the API of the corresponding
items in standard :mod:`socket` module exactly, but the synchronous functions
in this module only block the current greenlet and let the others run.

For convenience, exceptions (like :class:`error <socket.error>` and :class:`timeout <socket.timeout>`)
as well as the constants from :mod:`socket` module are imported into this module.
"""
import sys
def is_py3():
    return sys.hexversion >=  0x03000000

__implements__ = ['getaddrinfo',
                  'gethostbyname',
                  'socket',
                  'SocketType',
                  'fromfd',
                  'socketpair']

# non-standard functions that this module provides:
__extensions__ = ['wait_read',
                  'wait_write',
                  'wait_readwrite']

# standard functions and classes that this module re-imports
__imports__ = ['error',
               'gaierror',
               'getfqdn',
               'herror',
               'htonl',
               'htons',
               'ntohl',
               'ntohs',
               'inet_aton',
               'inet_ntoa',
               'inet_pton',
               'inet_ntop',
               'timeout',
               'gethostname',
               'getprotobyname',
               'getservbyname',
               'getservbyport',
               'getdefaulttimeout',
               'setdefaulttimeout',
               # Python 2.5 and older:
               'RAND_add',
               'RAND_egd',
               'RAND_status',
               ]

patched = True

import sys
import time
import random
import re

from errno import EINVAL
from errno import EWOULDBLOCK
from errno import EINPROGRESS
from errno import EALREADY
from errno import EAGAIN
from errno import EISCONN
from os import strerror

try:
    from errno import EBADF
except ImportError:
    EBADF = 9

import _socket
error = _socket.error
timeout = _socket.timeout
__socket__ = __import__('socket')
if is_py3():
    _realsocket = _socket.socket
else:
    _realsocket = _socket.socket
    _fileobject = __socket__._fileobject
gaierror = _socket.gaierror

gethostbyname = _socket.gethostbyname
getaddrinfo = _socket.getaddrinfo


for name in __imports__[:]:
    try:
        value = getattr(__socket__, name)
        globals()[name] = value
    except AttributeError:
        __imports__.remove(name)

if is_py3():
    for name in __socket__.__all__:
        value = getattr(__socket__, name)
        if isinstance(value, (int, str)):
            globals()[name] = value
            __imports__.append(name)
else:
    for name in __socket__.__all__:
        value = getattr(__socket__, name)
        if isinstance(value, (int, long, basestring)):
            globals()[name] = value
            __imports__.append(name)

del name, value

SOCKETMETHODS = ('bind', 'fileno', 'listen', 'getpeername', 'getsockname', 'getsockopt', 'setsockopt')

if 'inet_ntop' not in globals():
    # inet_ntop is required by our implementation of getaddrinfo

    def inet_ntop(address_family, packed_ip):
        if address_family == AF_INET:
            return inet_ntoa(packed_ip)
        # XXX: ipv6 won't work on windows
        raise NotImplementedError('inet_ntop() is not available on this platform')

from meinheld import server, cancel_wait


def wait_read(fileno, timeout=None):
    if not timeout:
        timeout = 0
    server.trampoline(fileno, read=True, timeout=int(timeout))

def wait_write(fileno, timeout=None):
    if not timeout:
        timeout = 0
    server.trampoline(fileno, write=True, timeout=int(timeout))

def wait_readwrite(fileno, timeout=None):
    if not timeout:
        timeout = 0
    server.trampoline(fileno, read=True, write=True, timeout=int(timeout))



if sys.version_info[:2] < (2, 7):
    _get_memory = buffer
else:
    def _get_memory(string, offset):
        return memoryview(string)[offset:]


class _closedsocket(object):
    __slots__ = []

    def _dummy(*args):
        raise error(EBADF, 'Bad file descriptor')
    # All _delegate_methods must also be initialized here.
    send = recv = recv_into = sendto = recvfrom = recvfrom_into = _dummy
    __getattr__ = _dummy


_delegate_methods = ("recv", "recvfrom", "recv_into", "recvfrom_into", "send", "sendto", 'sendall')

timeout_default = object()

def internal_accept(s):
    sock = s._sock
    while True:
        try:
            client_socket, address = sock.accept()
            break
        except error as ex:
            if ex.errno != EWOULDBLOCK or s.timeout == 0.0:
                raise
            #sys.exc_clear()
        wait_read(sock.fileno(), timeout=self.timeout)
    return socket(_sock=client_socket), address

def internal_close(s):
    cancel_wait(s._sock.fileno())
    s._sock = _closedsocket()
    dummy = s._sock._dummy
    for method in _delegate_methods:
        setattr(s, method, dummy)

def internal_connect(s, address):
    if s.timeout == 0.0:
        return s._sock.connect(address)
    sock = s._sock
    if s.timeout is None:
        while True:
            err = sock.getsockopt(SOL_SOCKET, SO_ERROR)
            if err:
                raise error(err, strerror(err))
            result = sock.connect_ex(address)
            if not result or result == EISCONN:
                break
            elif (result in (EWOULDBLOCK, EINPROGRESS, EALREADY)) or (result == EINVAL and is_windows):
                wait_readwrite(sock.fileno())
            else:
                raise error(result, strerror(result))
    else:
        end = time.time() + s.timeout
        while True:
            err = sock.getsockopt(SOL_SOCKET, SO_ERROR)
            if err:
                raise error(err, strerror(err))
            result = sock.connect_ex(address)
            if not result or result == EISCONN:
                break
            elif (result in (EWOULDBLOCK, EINPROGRESS, EALREADY)) or (result == EINVAL and is_windows):
                timeleft = end - time.time()
                if timeleft <= 0:
                    raise timeout('timed out')
                wait_readwrite(sock.fileno(), timeout=timeleft)
            else:
                raise error(result, strerror(result))

def internal_connect_ex(s, address):
    try:
        return s.connect(address) or 0
    except timeout:
        return EAGAIN
    except error as ex:
        if type(ex) is error:
            return ex.errno
        else:
            raise # gaierror is not silented by connect_ex

def internal_recv(s, *args):
    # print("internal_recv")
    sock = s._sock # keeping the reference so that fd is not closed during waiting
    while True:
        try:
            return sock.recv(*args)
        except error as ex:
            if ex.errno == EBADF:
                raise 
            if ex.errno != EWOULDBLOCK or s.timeout == 0.0:
                raise
            # QQQ without clearing exc_info test__refcount.test_clean_exit fails
            #sys.exc_clear()
        try:
            wait_read(sock.fileno(), timeout=s.timeout)
        except error as ex:
            if ex.errno == EBADF:
                return ''
            raise

def internal_recvfrom(s, *args):
    # print("internal_recvfrom")
    sock = s._sock
    while True:
        try:
            return sock.recvfrom(*args)
        except error as ex:
            if ex.errno != EWOULDBLOCK or s.timeout == 0.0:
                raise
            #sys.exc_clear()
        wait_read(sock.fileno(), timeout=s.timeout)

def internal_recvfrom_into(s, *args):
    # print("internal_recvfrom_into")
    sock = s._sock
    while True:
        try:
            return sock.recvfrom_into(*args)
        except error as ex:
            if ex.errno != EWOULDBLOCK or s.timeout == 0.0:
                raise
            #sys.exc_clear()
        wait_read(sock.fileno(), timeout=s.timeout)

def internal_recv_into(s, *args):
    # print("internal_recv_into")
    sock = s._sock
    while True:
        try:
            return sock.recv_into(*args)
        except error as ex:
            if ex.errno == EBADF:
                return 0
            if ex.errno != EWOULDBLOCK or s.timeout == 0.0:
                raise
            #sys.exc_clear()
        try:
            wait_read(sock.fileno(), timeout=s.timeout)
        except error as ex:
            if ex.errno == EBADF:
                return 0
            raise

def internal_send(s, data, flags=0, timeout=timeout_default):
    sock = s._sock
    if timeout is timeout_default:
        timeout = s.timeout
    try:
        return sock.send(data, flags)
    except error as ex:
        if ex.errno != EWOULDBLOCK or timeout == 0.0:
            raise
        #sys.exc_clear()
        try:
            wait_write(sock.fileno(), timeout=timeout)
        except error as ex:
            if ex.errno == EBADF:
                return 0
            raise
        try:
            return sock.send(data, flags)
        except error as ex2:
            if ex2.errno == EWOULDBLOCK:
                return 0
            raise

if is_py3():
    def internal_sendall(s, data, flags=0):
        # this sendall is also reused by SSL subclasses (both from ssl and sslold modules),
        # so it should not call s._sock methods directly
        if s.timeout is None:
            data_sent = 0
            while data_sent < len(data):
                data_sent += s.send(_get_memory(data, data_sent), flags)
        else:
            timeleft = s.timeout
            end = time.time() + timeleft
            data_sent = 0
            while True:
                data_sent += s.send(_get_memory(data, data_sent), flags, timeout=timeleft)
                if data_sent >= len(data):
                    break
                timeleft = end - time.time()
                if timeleft <= 0:
                    raise timeout('timed out')
else:
    def internal_sendall(s, data, flags=0):
        if isinstance(data, unicode):
            data = data.encode()
        # this sendall is also reused by SSL subclasses (both from ssl and sslold modules),
        # so it should not call s._sock methods directly
        if s.timeout is None:
            data_sent = 0
            while data_sent < len(data):
                data_sent += s.send(_get_memory(data, data_sent), flags)
        else:
            timeleft = s.timeout
            end = time.time() + timeleft
            data_sent = 0
            while True:
                data_sent += s.send(_get_memory(data, data_sent), flags, timeout=timeleft)
                if data_sent >= len(data):
                    break
                timeleft = end - time.time()
                if timeleft <= 0:
                    raise timeout('timed out')

def internal_sendto(s, *args):
    sock = s._sock
    try:
        return sock.sendto(*args)
    except error as ex:
        if ex.errno != EWOULDBLOCK or timeout == 0.0:
            raise
        #sys.exc_clear()
        wait_write(sock.fileno(), timeout=s.timeout)
        try:
            return sock.sendto(*args)
        except error as ex2:
            if ex2.errno == EWOULDBLOCK:
                return 0
            raise

def internal_settimeout(s, howlong):
    if howlong is not None:
        try:
            f = howlong.__float__
        except AttributeError:
            raise TypeError('a float is required')
        howlong = f()
        if howlong < 0.0:
            raise ValueError('Timeout value out of range')
    s.timeout = howlong

def internal_gettimeout(s):
    return s.timeout

def internal_shutdown(s, how):
    cancel_wait(s._sock.fileno())
    s._sock.shutdown(how)

if is_py3():
    class socket(object):
        
        patched = True
        #__slots__ = ["__weakref__", "_io_refs", "_closed", "_sock", "timeout"]
        
        def __init__(self, family=AF_INET, type=SOCK_STREAM, proto=0, fileno=None):
            self._sock = _socket.socket(family, type, proto, fileno)
            self._io_refs = 0
            self._closed = False
            self._sock.setblocking(0)
            self.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
            self.timeout = _socket.getdefaulttimeout()

        def __enter__(self):
            return self

        def __exit__(self, *args):
            if not self._closed:
                self.close()

        def dup(self):
            """dup() -> socket object

            Return a new socket object connected to the same system resource.
            Note, that the new socket does not inherit the timeout."""
            fd = dup(self.fileno())
            sock = self.__class__(self.family, self.type, self.proto, fileno=fd)
            sock.settimeout(self.gettimeout())
            return sock
        
        def _decref_socketios(self):
            if self._io_refs > 0:
                self._io_refs -= 1
            if self._closed:
                self.close()
        
        def _real_close(self, _ss=_socket.socket):
            # This function should not reference any globals. See issue #808164.
            _ss.close(self._sock)

        def close(self):
            # This function should not reference any globals. See issue #808164.
            self._closed = True
            if self._io_refs <= 0:
                self._real_close()

        def detach(self):
            """detach() -> file descriptor

            Close the socket object without closing the underlying file descriptor.
            The object cannot be used after this call, but the file descriptor
            can be reused for other purposes.  The file descriptor is returned.
            """
            self._closed = True
            return self._sock.detach()
        
        def accept(self):
            raise NotImplementedError()

        makefile = __socket__.socket.makefile
        connect = internal_connect
        connect_ex = internal_connect_ex
        recv = internal_recv
        recvfrom = internal_recvfrom
        recvfrom_into = internal_recvfrom_into
        recv_into = internal_recv_into
        send = internal_send
        sendall = internal_sendall
        sendto = internal_sendto
        settimeout = internal_settimeout
        gettimeout = internal_gettimeout
        shutdown = internal_shutdown

        family = property(lambda self: self._sock.family, doc="the socket family")
        type = property(lambda self: self._sock.type, doc="the socket type")
        proto = property(lambda self: self._sock.proto, doc="the socket protocol")
        _s = ("def %s(self, *args): return self._sock.%s(*args)\n\n"
              "%s.__doc__ = _realsocket.%s.__doc__\n")

        for _m in SOCKETMETHODS:
            exec(_s % (_m, _m, _m, _m))
        del _m, _s
else:
    class socket(object):
        
        patched = True

        def __init__(self, family=AF_INET, type=SOCK_STREAM, proto=0, _sock=None):
            if _sock is None:
                self._sock = _realsocket(family, type, proto)
                self.timeout = _socket.getdefaulttimeout()
            else:
                if hasattr(_sock, '_sock'):
                    self._sock = _sock._sock
                    self.timeout = getattr(_sock, 'timeout', False)
                    if self.timeout is False:
                        self.timeout = _socket.getdefaulttimeout()
                else:
                    self._sock = _sock
                    self.timeout = _socket.getdefaulttimeout()
            self._sock.setblocking(0)
            self.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)

        def __repr__(self):
            return '<%s at %s %s>' % (type(self).__name__, hex(id(self)), self._formatinfo())

        def __str__(self):
            return '<%s %s>' % (type(self).__name__, self._formatinfo())

        def _formatinfo(self):
            try:
                fileno = self.fileno()
            except Exception as ex:
                fileno = str(ex)
            try:
                sockname = self.getsockname()
                sockname = '%s:%s' % sockname
            except Exception:
                sockname = None
            try:
                peername = self.getpeername()
                peername = '%s:%s' % peername
            except Exception:
                peername = None
            result = 'fileno=%s' % fileno
            if sockname is not None:
                result += ' sock=' + str(sockname)
            if peername is not None:
                result += ' peer=' + str(peername)
            if self.timeout is not None:
                result += ' timeout=' + str(self.timeout)
            return result

        def __enter__(self):
            return self

        def __exit__(self, *args):
            if not self._closed:
                self.close()

        def dup(self):
            """dup() -> socket object

            Return a new socket object connected to the same system resource.
            Note, that the new socket does not inherit the timeout."""
            return socket(_sock=self._sock)

        def makefile(self, mode='r', bufsize=-1):
            # note that this does not inherit timeout either (intentionally, because that's
            # how the standard socket behaves)
            return _fileobject(self.dup(), mode, bufsize)
        
        def accept(self):
            raise NotImplementedError()
        close = internal_close
        connect = internal_connect
        connect_ex = internal_connect_ex
        recv = internal_recv
        recvfrom = internal_recvfrom
        recvfrom_into = internal_recvfrom_into
        recv_into = internal_recv_into
        send = internal_send
        sendall = internal_sendall
        sendto = internal_sendto
        settimeout = internal_settimeout
        gettimeout = internal_gettimeout
        shutdown = internal_shutdown

        family = property(lambda self: self._sock.family, doc="the socket family")
        type = property(lambda self: self._sock.type, doc="the socket type")
        proto = property(lambda self: self._sock.proto, doc="the socket protocol")

        # delegate the functions that we haven't implemented to the real socket object

        _s = ("def %s(self, *args): return self._sock.%s(*args)\n\n"
              "%s.__doc__ = _realsocket.%s.__doc__\n")

        for _m in SOCKETMETHODS:
            exec(_s % (_m, _m, _m, _m))
        del _m, _s

SocketType = socket

if hasattr(_socket, 'socketpair'):
    
    def socketpair(*args):
        one, two = _socket.socketpair(*args)
        return socket(_sock=one), socket(_sock=two)
else:
    __all__.remove('socketpair')

if is_py3():
    pass
else:
    if hasattr(_socket, 'fromfd'):
        
        def fromfd(*args):
            return socket(_sock=_socket.fromfd(*args))
    else:
        __all__.remove('fromfd')




try:
    _GLOBAL_DEFAULT_TIMEOUT = __socket__._GLOBAL_DEFAULT_TIMEOUT
except AttributeError:
    _GLOBAL_DEFAULT_TIMEOUT = object()

_have_ssl = False

# if is_py3():
    # pass
# else:
    # try:
        # from meinheld.ssl import sslwrap_simple as ssl, SSLError as sslerror
        # _have_ssl = True
    # except ImportError:
        # try:
            # from meinheld.sslold import ssl, sslerror
            # _have_ssl = True
        # except ImportError:
            # pass

    # if sys.version_info[:2] <= (2, 5) and _have_ssl:
        # __implements__.extend(['ssl', 'sslerror', 'SSLType'])


__all__ = __implements__ + __extensions__ + __imports__
